use serde::{Deserialize, Serialize};

#[derive(Serialize, Deserialize, Debug)]
pub struct Coin {
    amount: [u8; 16],
    denom: cosmrs::Denom,
}

impl From<&cosmrs::Coin> for Coin {
    fn from(value: &cosmrs::Coin) -> Self {
        let cosmrs::Coin { denom, amount } = value;
        Coin {
            denom: denom.clone(),
            amount: amount.to_be_bytes(),
        }
    }
}

impl From<&Coin> for cosmrs::Coin {
    fn from(value: &Coin) -> Self {
        let Coin { amount, denom } = value;
        cosmrs::Coin {
            denom: denom.clone(),
            amount: u128::from_be_bytes(*amount),
        }
    }
}

#[derive(Serialize, Deserialize, Debug)]
pub struct Fee {
    amount: Vec<Coin>,
    gas_limit: u64,
}

impl From<&cosmrs::tx::Fee> for Fee {
    fn from(value: &cosmrs::tx::Fee) -> Self {
        let cosmrs::tx::Fee {
            amount,
            gas_limit,
            payer,
            granter,
        } = value;

        assert!(payer.is_none(), "unimplemented: payer");
        assert!(granter.is_none(), "unimplemented: granter");

        let amounts = amount.iter().map(Coin::from).collect::<Vec<_>>();

        Fee {
            amount: amounts,
            gas_limit: *gas_limit,
        }
    }
}

impl From<&Fee> for cosmrs::tx::Fee {
    fn from(value: &Fee) -> Self {
        let Fee { amount, gas_limit } = value;
        let amounts = amount.iter().map(cosmrs::Coin::from).collect::<Vec<_>>();

        cosmrs::tx::Fee {
            amount: amounts,
            gas_limit: *gas_limit,
            payer: None,
            granter: None,
        }
    }
}
